@@ -21,57 +21,58 @@
 #include <glib/gprintf.h>
 #include <gtk/gtk.h>
 #include <gtk/gtkimmodule.h>
-#include "ibusimclient.h"
+#include <ibus.h>
 #include "ibusimcontext.h"
 
 #define IBUS_LOCALDIR ""
 static const GtkIMContextInfo ibus_im_info = {
     "ibus",
-    "iBus (Intelligent Input Bus)",
+    "IBus (Intelligent Input Bus)",
     "ibus",
     IBUS_LOCALDIR,
     "ja:ko:zh"
 };
 
-static const GtkIMContextInfo * info_list[] = {
+static const GtkIMContextInfo *info_list[] = {
     &ibus_im_info
 };
 
-
-void
-im_module_init (GTypeModule *type_module)
+G_MODULE_EXPORT const gchar*
+g_module_check_init (GModule *module)
 {
-    ibus_im_client_register_type(type_module);
-    ibus_im_context_register_type(type_module);
-#if 0
-    gchar **p = environ;
-    extern gchar **environ;
-    while (*p != NULL) {
-        g_fprintf (stderr, "%s\n", *p);
-        p ++;
-    }
-#endif
+    return glib_check_version (GLIB_MAJOR_VERSION,
+                               GLIB_MINOR_VERSION,
+                               GLIB_MICRO_VERSION);
 }
 
-void
+G_MODULE_EXPORT void
+im_module_init (GTypeModule *type_module)
+{
+    /* make module resident */
+    g_type_module_use (type_module);
+
+    ibus_im_context_register_type (type_module);
+}
+
+G_MODULE_EXPORT void
 im_module_exit (void)
 {
 }
 
-GtkIMContext *
+G_MODULE_EXPORT GtkIMContext *
 im_module_create (const gchar *context_id)
 {
     if (g_strcmp0 (context_id, "ibus") == 0) {
         IBusIMContext *context;
         context = ibus_im_context_new ();
-        return GTK_IM_CONTEXT(context);
+        return (GtkIMContext *) context;
     }
     return NULL;
 }
 
-void
+G_MODULE_EXPORT void
 im_module_list (const GtkIMContextInfo ***contexts,
-                int                      *n_contexts)
+                gint                     *n_contexts)
 {
     *contexts = info_list;
     *n_contexts = G_N_ELEMENTS (info_list);
