@@ -44,60 +44,71 @@
         g_debug (fmt_args); \
     }
 
-#include <ibusattribute.h>
-#include <ibusimclient.h>
+#include <ibus.h>
 #include "gdk-private.h"
+#include "locales.h"
 
 struct _X11ICONN {
     GList        *clients;
 };
 typedef struct _X11ICONN    X11ICONN;
 
+typedef struct _X11IC    X11IC;
 struct _X11IC {
-    gchar       *ibus_ic;
-    Window      client_window;
-    Window      focus_window;
-    gint32      input_style;
-    X11ICONN    *conn;
-    gint        icid;
-    gint        connect_id;
-    gchar       *lang;
-    gboolean        has_preedit_area;
-    GdkRectangle    preedit_area;
+    IBusInputContext *context;
+    Window           client_window;
+    Window           focus_window;
+    gint32           input_style;
+    X11ICONN        *conn;
+    gint             icid;
+    gint             connect_id;
+    gchar           *lang;
+    gboolean         has_preedit_area;
+    GdkRectangle     preedit_area;
 
     gchar           *preedit_string;
     IBusAttrList    *preedit_attrs;
-    gint            preedit_cursor;
-    gboolean        preedit_visible;
-    gboolean        preedit_started;
-    gint            onspot_preedit_length;
+    gint             preedit_cursor;
+    gboolean         preedit_visible;
+    gboolean         preedit_started;
+    gint             onspot_preedit_length;
 };
 
-typedef struct _X11IC    X11IC;
+static void     _xim_set_cursor_location    (X11IC              *x11ic);
+static void     _context_commit_text_cb     (IBusInputContext   *context,
+                                             IBusText           *text,
+                                             X11IC              *x11ic);
+static void     _context_forward_key_event_cb
+                                            (IBusInputContext   *context,
+                                             X11IC              *x11ic);
 
-static void _xim_set_cursor_location (X11IC *x11ic);
+static void     _context_update_preedit_text_cb
+                                            (IBusInputContext   *context,
+                                             IBusText           *text,
+                                             gint                cursor_pos,
+                                             gboolean            visible,
+                                             X11IC              *x11ic);
+static void     _context_show_preedit_text_cb
+                                            (IBusInputContext   *context,
+                                             X11IC              *x11ic);
+static void     _context_hide_preedit_text_cb
+                                            (IBusInputContext   *context,
+                                             X11IC              *x11ic);
+static void     _context_enabled_cb         (IBusInputContext   *context,
+                                             X11IC              *x11ic);
+static void     _context_disabled_cb        (IBusInputContext   *context,
+                                             X11IC              *x11ic);
 
 static GHashTable     *_x11_ic_table = NULL;
-static GHashTable     *_ibus_ic_table = NULL;
 static GHashTable     *_connections = NULL;
 static XIMS _xims = NULL;
 static gchar _server_name[128] = "ibus";
-static gchar _locale[1024] =
-    "aa,af,am,an,ar,as,az,be,bg,bn,br,bs,"
-    "ca,cs,cy,da,de,dz,el,en,es,et,eu,"
-    "fa,fi,fo,fr,fy,ga,gd,gl,gu,gv,"
-    "he,hi,hr,hu,hy,id,is,it,iw,ja,"
-    "ka,kk,kl,km,kn,ko,ku,kw,ky,lg,lo,lt,lv,"
-    "mg,mi,mk,ml,mn,mr,ms,mt,nb,ne,nl,nn,no,nr,"
-    "oc,om,or,pa,pl,pt,ro,ru,rw,"
-    "se,si,sk,sl,so,sq,sr,ss,st,sv,"
-    "ta,te,tg,th,ti,tl,tn,tr,ts,tt,"
-    "uk,ur,uz,ve,vi,wa,xh,yi,zh,zu";
+static gchar _locale[1024] = LOCALES_STRING;
 
 static gboolean _kill_daemon = FALSE;
-static gint        g_debug_level = 0;
+static gint     g_debug_level = 0;
 
-static IBusIMClient *_client = NULL;
+static IBusBus *_bus = NULL;
 
 static void
 _xim_preedit_start (XIMS xims, const X11IC *x11ic)
@@ -307,7 +318,7 @@ xim_create_ic (XIMS xims, IMChangeICStruct *call_data)
     x11ic->icid = call_data->icid;
     x11ic->connect_id = call_data->connect_id;
     x11ic->conn = (X11ICONN *)g_hash_table_lookup (_connections,
-                        (gconstpointer)(unsigned long)call_data->connect_id);
+                                                   GINT_TO_POINTER ((gint) call_data->connect_id));
     if (x11ic->conn == NULL) {
         g_slice_free (X11IC, x11ic);
         g_return_val_if_reached (0);
@@ -315,27 +326,41 @@ xim_create_ic (XIMS xims, IMChangeICStruct *call_data)
 
     i = _xim_store_ic_values (x11ic, call_data);
 
-    x11ic->ibus_ic = g_strdup (ibus_im_client_create_input_context (_client));
-    if (x11ic->ibus_ic == NULL) {
+    x11ic->context = ibus_bus_create_input_context (_bus, "xim");
+
+    if (x11ic->context == NULL) {
         g_slice_free (X11IC, x11ic);
         g_return_val_if_reached (0);
     }
 
-    g_hash_table_insert (_ibus_ic_table, x11ic->ibus_ic, (gpointer)x11ic);
+    g_signal_connect (x11ic->context, "commit-text",
+                        G_CALLBACK (_context_commit_text_cb), x11ic);
+    g_signal_connect (x11ic->context, "forward-key-event",
+                        G_CALLBACK (_context_forward_key_event_cb), x11ic);
+
+    g_signal_connect (x11ic->context, "update-preedit-text",
+                        G_CALLBACK (_context_update_preedit_text_cb), x11ic);
+    g_signal_connect (x11ic->context, "show-preedit-text",
+                        G_CALLBACK (_context_show_preedit_text_cb), x11ic);
+    g_signal_connect (x11ic->context, "hide-preedit-text",
+                        G_CALLBACK (_context_hide_preedit_text_cb), x11ic);
+    g_signal_connect (x11ic->context, "enabled",
+                        G_CALLBACK (_context_enabled_cb), x11ic);
+    g_signal_connect (x11ic->context, "disabled",
+                        G_CALLBACK (_context_disabled_cb), x11ic);
+
 
     if (x11ic->input_style & XIMPreeditCallbacks) {
-        ibus_im_client_set_capabilities (_client, x11ic->ibus_ic,
-                IBUS_CAP_FOCUS | IBUS_CAP_PREEDIT);
+        ibus_input_context_set_capabilities (x11ic->context, IBUS_CAP_FOCUS | IBUS_CAP_PREEDIT_TEXT);
     }
     else {
-        ibus_im_client_set_capabilities (_client, x11ic->ibus_ic,
-                IBUS_CAP_FOCUS);
+        ibus_input_context_set_capabilities (x11ic->context, IBUS_CAP_FOCUS);
     }
 
     g_hash_table_insert (_x11_ic_table,
-                                (gpointer)x11ic->icid, (gpointer)x11ic);
+                         GINT_TO_POINTER (x11ic->icid), (gpointer)x11ic);
     x11ic->conn->clients = g_list_append (x11ic->conn->clients,
-                                (gpointer)x11ic);
+                         (gpointer)x11ic);
     return 1;
 }
 
@@ -349,19 +374,26 @@ xim_destroy_ic (XIMS xims, IMChangeICStruct *call_data)
                 call_data->icid, call_data->connect_id);
 
     x11ic = (X11IC *)g_hash_table_lookup (_x11_ic_table,
-                (gconstpointer)(unsigned long)call_data->icid);
+                                          GINT_TO_POINTER ((gint) call_data->icid));
     g_return_val_if_fail (x11ic != NULL, 0);
 
-    ibus_im_client_release_input_context (_client, x11ic->ibus_ic);
+    if (x11ic->context) {
+        ibus_object_destroy ((IBusObject *)x11ic->context);
+        g_object_unref (x11ic->context);
+        x11ic->context = NULL;
+    }
 
-    g_hash_table_remove (_ibus_ic_table, x11ic->ibus_ic);
     g_hash_table_remove (_x11_ic_table,
-                (gconstpointer)(unsigned long)call_data->icid);
+                         GINT_TO_POINTER ((gint) call_data->icid));
     x11ic->conn->clients = g_list_remove (x11ic->conn->clients, (gconstpointer)x11ic);
 
     g_free (x11ic->preedit_string);
-    ibus_attr_list_unref (x11ic->preedit_attrs);
-    g_free (x11ic->ibus_ic);
+    x11ic->preedit_string = NULL;
+
+    if (x11ic->preedit_attrs) {
+        g_object_unref (x11ic->preedit_attrs);
+        x11ic->preedit_attrs = NULL;
+    }
 
     g_slice_free (X11IC, x11ic);
 
@@ -376,11 +408,11 @@ xim_set_ic_focus (XIMS xims, IMChangeFocusStruct *call_data)
     LOG (1, "XIM_SET_IC_FOCUS ic=%d connect_id=%d",
                 call_data->icid, call_data->connect_id);
 
-    x11ic = (X11IC *)g_hash_table_lookup (_x11_ic_table,
-                (gconstpointer)(unsigned long)call_data->icid);
+    x11ic = (X11IC *) g_hash_table_lookup (_x11_ic_table,
+                                           GINT_TO_POINTER ((gint) call_data->icid));
     g_return_val_if_fail (x11ic != NULL, 0);
 
-    ibus_im_client_focus_in (_client, x11ic->ibus_ic);
+    ibus_input_context_focus_in (x11ic->context);
     _xim_set_cursor_location (x11ic);
 
     return 1;
@@ -394,11 +426,11 @@ xim_unset_ic_focus (XIMS xims, IMChangeFocusStruct *call_data)
     LOG (1, "XIM_UNSET_IC_FOCUS ic=%d connect_id=%d",
                 call_data->icid, call_data->connect_id);
 
-    x11ic = (X11IC *)g_hash_table_lookup (_x11_ic_table,
-            (gconstpointer)(unsigned long)call_data->icid);
+    x11ic = (X11IC *) g_hash_table_lookup (_x11_ic_table,
+                                           GINT_TO_POINTER ((gint) call_data->icid));
     g_return_val_if_fail (x11ic != NULL, 0);
 
-    ibus_im_client_focus_out (_client, x11ic->ibus_ic);
+    ibus_input_context_focus_out (x11ic->context);
 
     return 1;
 
@@ -414,8 +446,8 @@ xim_forward_event (XIMS xims, IMForwardEventStruct *call_data)
     LOG (1, "XIM_FORWARD_EVENT ic=%d connect_id=%d",
                 call_data->icid, call_data->connect_id);
 
-    x11ic = (X11IC *)g_hash_table_lookup (_x11_ic_table,
-                (gconstpointer)(unsigned long)call_data->icid);
+    x11ic = (X11IC *) g_hash_table_lookup (_x11_ic_table,
+                                           GINT_TO_POINTER ((gint) call_data->icid));
     g_return_val_if_fail (x11ic != NULL, 0);
 
     xevent = (XKeyEvent*) &(call_data->event);
@@ -426,8 +458,12 @@ xim_forward_event (XIMS xims, IMForwardEventStruct *call_data)
     event.send_event = xevent->send_event;
     event.window = NULL;
 
-    if (ibus_im_client_filter_keypress (_client,
-                    x11ic->ibus_ic, &event, False)) {
+    if (event.type == GDK_KEY_RELEASE) {
+        event.state |= IBUS_RELEASE_MASK;
+    }
+
+    if (ibus_input_context_process_key_event (x11ic->context,
+                event.keyval, event.state)) {
         if (! x11ic->has_preedit_area) {
             _xim_set_cursor_location (x11ic);
         }
@@ -458,8 +494,8 @@ xim_open (XIMS xims, IMOpenStruct *call_data)
     LOG (1, "XIM_OPEN connect_id=%d",
                 call_data->connect_id);
 
-    conn = (X11ICONN *)g_hash_table_lookup (_connections,
-                (gconstpointer)(unsigned long)call_data->connect_id);
+    conn = (X11ICONN *) g_hash_table_lookup (_connections,
+                                             GINT_TO_POINTER ((gint) call_data->connect_id));
     g_return_val_if_fail (conn == NULL, 0);
 
     conn = g_slice_new0 (X11ICONN);
@@ -479,14 +515,21 @@ _free_ic (gpointer data, gpointer user_data)
     g_return_if_fail (x11ic != NULL);
 
     g_free (x11ic->preedit_string);
-    ibus_attr_list_unref (x11ic->preedit_attrs);
-    g_free (x11ic->ibus_ic);
+
+    if (x11ic->preedit_attrs) {
+        g_object_unref (x11ic->preedit_attrs);
+        x11ic->preedit_attrs = NULL;
+    }
+
+    if (x11ic->context) {
+        ibus_object_destroy ((IBusObject *)x11ic->context);
+        g_object_unref (x11ic->context);
+        x11ic->context = NULL;
+    }
 
     /* Remove the IC from g_client dictionary */
-    g_hash_table_remove (_ibus_ic_table,
-                (gconstpointer)(unsigned long)x11ic->ibus_ic);
     g_hash_table_remove (_x11_ic_table,
-                (gconstpointer)(unsigned long)x11ic->icid);
+                         GINT_TO_POINTER (x11ic->icid));
 
     g_slice_free (X11IC, x11ic);
 }
@@ -499,8 +542,8 @@ xim_close (XIMS ims, IMCloseStruct *call_data)
     LOG (1, "XIM_CLOSE connect_id=%d",
                 call_data->connect_id);
 
-    conn = (X11ICONN *)g_hash_table_lookup (_connections,
-                (gconstpointer)(unsigned long)call_data->connect_id);
+    conn = (X11ICONN *) g_hash_table_lookup (_connections,
+                                             GINT_TO_POINTER ((gint) call_data->connect_id));
     g_return_val_if_fail (conn != NULL, 0);
 
     g_list_foreach (conn->clients, _free_ic, NULL);
@@ -508,7 +551,7 @@ xim_close (XIMS ims, IMCloseStruct *call_data)
     g_list_free (conn->clients);
 
     g_hash_table_remove (_connections,
-            (gconstpointer)(unsigned long)call_data->connect_id);
+                         GINT_TO_POINTER ((gint) call_data->connect_id));
 
     g_slice_free (X11ICONN, conn);
 
@@ -551,8 +594,11 @@ _xim_set_cursor_location (X11IC *x11ic)
         }
     }
 
-    ibus_im_client_set_cursor_location (_client,
-            x11ic->ibus_ic, &preedit_area);
+    ibus_input_context_set_cursor_location (x11ic->context,
+            preedit_area.x,
+            preedit_area.y,
+            preedit_area.width,
+            preedit_area.height);
 }
 
 
@@ -565,8 +611,8 @@ xim_set_ic_values (XIMS xims, IMChangeICStruct *call_data)
     LOG (1, "XIM_SET_IC_VALUES ic=%d connect_id=%d",
                 call_data->icid, call_data->connect_id);
 
-    x11ic = (X11IC *)g_hash_table_lookup (_x11_ic_table,
-                (gconstpointer)(unsigned long)call_data->icid);
+    x11ic = (X11IC *) g_hash_table_lookup (_x11_ic_table,
+                                           GINT_TO_POINTER ((gint) call_data->icid));
     g_return_val_if_fail (x11ic != NULL, 0);
 
     i = _xim_store_ic_values (x11ic, call_data);
@@ -587,8 +633,8 @@ xim_get_ic_values (XIMS xims, IMChangeICStruct *call_data)
     LOG (1, "XIM_GET_IC_VALUES ic=%d connect_id=%d",
                 call_data->icid, call_data->connect_id);
 
-    x11ic = (X11IC *)g_hash_table_lookup (_x11_ic_table,
-                (gconstpointer)(unsigned long)call_data->icid);
+    x11ic = (X11IC *) g_hash_table_lookup (_x11_ic_table,
+                                           GINT_TO_POINTER ((gint) call_data->icid));
     g_return_val_if_fail (x11ic != NULL, 0);
 
     XICAttribute *ic_attr = call_data->ic_attr;
@@ -614,11 +660,11 @@ xim_reset_ic (XIMS xims, IMResetICStruct *call_data)
     LOG (1, "XIM_RESET_IC ic=%d connect_id=%d",
                 call_data->icid, call_data->connect_id);
 
-    x11ic = (X11IC *)g_hash_table_lookup (_x11_ic_table,
-                (gconstpointer)(unsigned long)call_data->icid);
+    x11ic = (X11IC *) g_hash_table_lookup (_x11_ic_table,
+                                           GINT_TO_POINTER ((gint) call_data->icid));
     g_return_val_if_fail (x11ic != NULL, 0);
 
-    ibus_im_client_reset (_client, x11ic->ibus_ic);
+    ibus_input_context_reset (x11ic->context);
 
     return 1;
 }
@@ -668,7 +714,7 @@ ims_protocol_handler (XIMS xims, IMProtocol *call_data)
     }
 }
 
-
+/*
 static void
 _xim_forward_gdk_event (GdkEventKey *event, X11IC *x11ic)
 {
@@ -700,25 +746,32 @@ _xim_forward_gdk_event (GdkEventKey *event, X11IC *x11ic)
 
     IMForwardEvent (_xims, (XPointer) & fe);
 }
+*/
 
 static void
-_client_disconnected_cb (IBusIMClient *client, gpointer user_data)
+_bus_disconnected_cb (IBusBus  *bus,
+                      gpointer  user_data)
 {
     g_warning ("Connection closed by ibus-daemon");
+    g_object_unref (_bus);
+    _bus = NULL;
     exit(EXIT_SUCCESS);
 }
 
 static void
-_client_commit_string_cb (IBusIMClient *client, const gchar *ic, const gchar *string, gpointer user_data)
+_context_commit_text_cb (IBusInputContext *context,
+                         IBusText         *text,
+                         X11IC            *x11ic)
 {
-    X11IC *x11ic = g_hash_table_lookup (_ibus_ic_table, ic);
-    g_return_if_fail (x11ic != NULL);
+    g_assert (IBUS_IS_INPUT_CONTEXT (context));
+    g_assert (IBUS_IS_TEXT (text));
+    g_assert (x11ic != NULL);
 
     XTextProperty tp;
     IMCommitStruct cms = {0};
 
     Xutf8TextListToTextProperty (GDK_DISPLAY (),
-        (gchar **)&string, 1, XCompoundTextStyle, &tp);
+        (gchar **)&(text->text), 1, XCompoundTextStyle, &tp);
 
     cms.major_code = XIM_COMMIT;
     cms.icid = x11ic->icid;
@@ -731,12 +784,12 @@ _client_commit_string_cb (IBusIMClient *client, const gchar *ic, const gchar *st
 }
 
 static void
-_client_forward_event_cb (IBusIMClient *client, const gchar *ic, GdkEvent *event, gpointer user_data)
+_context_forward_key_event_cb (IBusInputContext *context,
+                               X11IC            *x11ic)
 {
-    X11IC *x11ic = g_hash_table_lookup (_ibus_ic_table, ic);
-    g_return_if_fail (x11ic != NULL);
+    g_assert (x11ic);
 
-    _xim_forward_gdk_event (&(event->key), x11ic);
+    // _xim_forward_gdk_event (&(event->key), x11ic);
 }
 
 static void
@@ -759,21 +812,27 @@ _update_preedit (X11IC *x11ic)
 }
 
 static void
-_client_update_preedit_cb (IBusIMClient *client, const gchar *ic, const gchar *string,
-    IBusAttrList *attrs, gint cursor_pos, gboolean visible, gpointer user_data)
+_context_update_preedit_text_cb (IBusInputContext *context,
+                                 IBusText         *text,
+                                 gint              cursor_pos,
+                                 gboolean          visible,
+                                 X11IC            *x11ic)
 {
-    X11IC *x11ic = g_hash_table_lookup (_ibus_ic_table, ic);
-    g_return_if_fail (x11ic != NULL);
+    g_assert (IBUS_IS_INPUT_CONTEXT (context));
+    g_assert (IBUS_IS_TEXT (text));
+    g_assert (x11ic);
 
     if (x11ic->preedit_string) {
         g_free(x11ic->preedit_string);
     }
-    x11ic->preedit_string = g_strdup(string);
+    x11ic->preedit_string = g_strdup(text->text);
 
     if (x11ic->preedit_attrs) {
-        ibus_attr_list_unref (x11ic->preedit_attrs);
+        g_object_unref (x11ic->preedit_attrs);
     }
-    x11ic->preedit_attrs = ibus_attr_list_ref(attrs);
+
+    g_object_ref(text->attrs);
+    x11ic->preedit_attrs = text->attrs;
 
     x11ic->preedit_cursor = cursor_pos;
     x11ic->preedit_visible = visible;
@@ -782,74 +841,62 @@ _client_update_preedit_cb (IBusIMClient *client, const gchar *ic, const gchar *s
 }
 
 static void
-_client_show_preedit_cb (IBusIMClient *client, const gchar *ic, gpointer user_data)
+_context_show_preedit_text_cb (IBusInputContext *context,
+                               X11IC            *x11ic)
 {
-    X11IC *x11ic = g_hash_table_lookup (_ibus_ic_table, ic);
-    g_return_if_fail (x11ic != NULL);
+    g_assert (IBUS_IS_INPUT_CONTEXT (context));
+    g_assert (x11ic);
 
     x11ic->preedit_visible = TRUE;
     _update_preedit (x11ic);
 }
 
 static void
-_client_hide_preedit_cb (IBusIMClient *client, const gchar *ic, gpointer user_data)
+_context_hide_preedit_text_cb (IBusInputContext *context,
+                               X11IC            *x11ic)
 {
-    X11IC *x11ic = g_hash_table_lookup (_ibus_ic_table, ic);
-    g_return_if_fail (x11ic != NULL);
+    g_assert (IBUS_IS_INPUT_CONTEXT (context));
+    g_assert (x11ic);
 
     x11ic->preedit_visible = FALSE;
     _update_preedit (x11ic);
 }
 
 static void
-_client_enabled_cb (IBusIMClient *client, const gchar *ic, gpointer user_data)
+_context_enabled_cb (IBusInputContext *context,
+                    X11IC            *x11ic)
 {
-    X11IC *x11ic = g_hash_table_lookup (_ibus_ic_table, ic);
-    g_return_if_fail (x11ic != NULL);
+    g_assert (IBUS_IS_INPUT_CONTEXT (context));
+    g_assert (x11ic);
 
     _xim_preedit_start (_xims, x11ic);
 }
 
 static void
-_client_disabled_cb (IBusIMClient *client, const gchar *ic, gpointer user_data)
+_context_disabled_cb (IBusInputContext *context,
+                    X11IC            *x11ic)
 {
-    X11IC *x11ic = g_hash_table_lookup (_ibus_ic_table, ic);
-    g_return_if_fail (x11ic != NULL);
+    g_assert (IBUS_IS_INPUT_CONTEXT (context));
+    g_assert (x11ic);
 
     _xim_preedit_end (_xims, x11ic);
 }
 
 static void
-_init_ibus_client (void)
+_init_ibus (void)
 {
-    if (_client != NULL)
+    if (_bus != NULL)
         return;
+    ibus_init ();
 
-    ibus_im_client_register_type (NULL);
+    _bus = ibus_bus_new ();
 
-    _client = ibus_im_client_new ();
-
-    if (!ibus_im_client_get_connected (_client)) {
+    if (!ibus_bus_is_connected (_bus)) {
         g_error ("Can not connect to ibus-daemon!");
     }
 
-    g_signal_connect (_client, "disconnected",
-                        G_CALLBACK (_client_disconnected_cb), NULL);
-    g_signal_connect (_client, "commit-string",
-                        G_CALLBACK (_client_commit_string_cb), NULL);
-    g_signal_connect (_client, "forward-event",
-                        G_CALLBACK (_client_forward_event_cb), NULL);
-
-    g_signal_connect (_client, "update-preedit",
-                        G_CALLBACK (_client_update_preedit_cb), NULL);
-    g_signal_connect (_client, "show-preedit",
-                        G_CALLBACK (_client_show_preedit_cb), NULL);
-    g_signal_connect (_client, "hide-preedit",
-                        G_CALLBACK (_client_hide_preedit_cb), NULL);
-    g_signal_connect (_client, "enabled",
-                        G_CALLBACK (_client_enabled_cb), NULL);
-    g_signal_connect (_client, "disabled",
-                        G_CALLBACK (_client_disabled_cb), NULL);
+    g_signal_connect (_bus, "disconnected",
+                        G_CALLBACK (_bus_disconnected_cb), NULL);
 }
 
 static void
@@ -881,11 +928,11 @@ _xim_init_IMdkit ()
     };
 
     GdkWindowAttr window_attr = {
-        title : "ibus-xim",
-        event_mask : GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK,
-        wclass : GDK_INPUT_OUTPUT,
-        window_type : GDK_WINDOW_TOPLEVEL,
-        override_redirect : 1,
+        .title              = "ibus-xim",
+        .event_mask         = GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK,
+        .wclass             = GDK_INPUT_OUTPUT,
+        .window_type        = GDK_WINDOW_TOPLEVEL,
+        .override_redirect   = 1,
     };
 
     XIMStyles styles;
@@ -915,9 +962,9 @@ _xim_init_IMdkit ()
         IMFilterEventMask, KeyPressMask | KeyReleaseMask,
         NULL);
 
-    _init_ibus_client ();
+    _init_ibus ();
 
-    if (!ibus_im_client_get_connected (_client)) {
+    if (!ibus_bus_is_connected (_bus)) {
         g_warning ("Can not connect to ibus daemon");
         exit (1);
     }
@@ -926,7 +973,9 @@ _xim_init_IMdkit ()
 static void
 _atexit_cb ()
 {
-    ibus_im_client_kill_daemon(_client);
+    if (_bus && ibus_bus_is_connected (_bus)) {
+        ibus_bus_exit(_bus, False);
+    }
 }
 
 static void
@@ -942,7 +991,8 @@ _print_usage (FILE *fp, gchar *name)
         "Usage:\n"
         " %s --help               Show this message\n"
         "    --server-name= -n    Setup xim sevrer name\n"
-        "    --locale= -l         Setup support locale\n"
+        "    --locale= -l         Setup support locales\n"
+        "    --locale-append= -a  Append locales into the default support locales\n"
         "    --kill-daemon -k     Kill ibus daemon when exit\n"
         "    --debug= -v          Setup debug level\n",
         name);
@@ -969,15 +1019,16 @@ main (int argc, char **argv)
 
     while (1) {
         static struct option long_options [] = {
-            {"debug", 1, 0, 0},
-            {"server-name", 1, 0, 0},
-            {"locale", 1, 0, 0},
-            {"help", 0, 0, 0},
-            {"kill-daemon", 0, 0, 0},
-            {0, 0, 0, 0},
+            { "debug", 1, 0, 0},
+            { "server-name", 1, 0, 0},
+            { "locale", 1, 0, 0},
+            { "locale-append", 1, 0, 0},
+            { "help", 0, 0, 0},
+            { "kill-daemon", 0, 0, 0},
+            { 0, 0, 0, 0},
         };
 
-        c = getopt_long (argc, argv, "v:n:l:k",
+        c = getopt_long (argc, argv, "v:n:l:k:a",
             long_options, &option_index);
 
         if (c == -1) break;
@@ -993,6 +1044,9 @@ main (int argc, char **argv)
             else if (g_strcmp0 (long_options[option_index].name, "locale") == 0) {
                 strncpy (_locale, optarg, sizeof (_locale));
             }
+            else if (g_strcmp0 (long_options[option_index].name, "locale-append") == 0) {
+                strncat (_locale, optarg, sizeof (_locale));
+            }
             else if (g_strcmp0 (long_options[option_index].name, "help") == 0) {
                 _print_usage (stdout, argv[0]);
                 exit (EXIT_SUCCESS);
@@ -1010,6 +1064,9 @@ main (int argc, char **argv)
         case 'l':
             strncpy (_locale, optarg, sizeof (_locale));
             break;
+        case 'a':
+            strncat (_locale, optarg, sizeof (_locale));
+            break;
         case 'k':
             _kill_daemon = TRUE;
             break;
@@ -1021,7 +1078,6 @@ main (int argc, char **argv)
     }
 
     _x11_ic_table = g_hash_table_new (g_direct_hash, g_direct_equal);
-    _ibus_ic_table = g_hash_table_new (g_str_hash, g_str_equal);
     _connections = g_hash_table_new (g_direct_hash, g_direct_equal);
 
     signal (SIGINT, _sighandler);
