@@ -21,14 +21,15 @@
 
 __all__ = (
         "IBUS_ADDR",
-        "IBUS_IFACE",
-        "IBUS_NAME",
-        "IBUS_PATH",
-        "IBUS_CONFIG_IFACE",
-        "IBUS_ENGINE_FACTORY_IFACE",
-        "IBUS_ENGINE_IFACE",
-        "IBUS_PANEL_IFACE",
-        "IBUS_NOTIFICATIONS_IFACE",
+        "IBUS_IFACE_IBUS",
+        "IBUS_SERVICE_IBUS",
+        "IBUS_PATH_IBUS",
+        "IBUS_IFACE_CONFIG",
+        "IBUS_IFACE_PANEL",
+        "IBUS_IFACE_ENGINE",
+        "IBUS_IFACE_ENGINE_FACTORY",
+        "IBUS_IFACE_INPUT_CONTEXT",
+        "IBUS_IFACE_NOTIFICATIONS",
         "default_reply_handler",
         "default_error_handler",
         "DEFAULT_ASYNC_HANDLERS",
@@ -46,9 +47,11 @@ __all__ = (
 import os
 import sys
 
-display = os.environ["DISPLAY"]
-if "." not in display:
-    display += ".0" 
+__display = os.environ["DISPLAY"]
+__hostname, __display_screen = __display.split(":", 1)
+if not __hostname.strip():
+    __hostname = "unix"
+__display_number = __display_screen.split(".")[0]
 
 __username = None
 try:
@@ -64,18 +67,24 @@ if not __username:
 if not __username:
     __username = os.getenv ("USERNAME")
 
-IBUS_ADDR = "unix:path=/tmp/ibus-%s/ibus-%s" % (__username, display.replace(":", "-"))
+__session_id = os.getenv ("IBUS_SESSION_ID")
+
+IBUS_ADDR = "unix:path=/tmp/ibus-%s%s/ibus-%s-%s" % (__username,
+                                                     "-" + __session_id if __session_id else "",
+                                                     __hostname,
+                                                     __display_number)
 # IBUS_ADDR  = "tcp:host=localhost,port=7799"
 
-IBUS_IFACE = "org.freedesktop.IBus"
-IBUS_PATH  = "/org/freedesktop/IBus"
-IBUS_NAME  = "org.freedesktop.IBus"
+IBUS_IFACE_IBUS     = "org.freedesktop.IBus"
+IBUS_PATH_IBUS      = "/org/freedesktop/IBus"
+IBUS_SERVICE_IBUS   = "org.freedesktop.IBus"
 
-IBUS_CONFIG_IFACE = "org.freedesktop.ibus.Config"
-IBUS_ENGINE_FACTORY_IFACE = "org.freedesktop.ibus.EngineFactory"
-IBUS_ENGINE_IFACE = "org.freedesktop.ibus.Engine"
-IBUS_PANEL_IFACE = "org.freedesktop.ibus.Panel"
-IBUS_NOTIFICATIONS_IFACE = "org.freedesktop.ibus.Notifications"
+IBUS_IFACE_PANEL            = "org.freedesktop.IBus.Panel"
+IBUS_IFACE_CONFIG           = "org.freedesktop.IBus.Config"
+IBUS_IFACE_ENGINE           = "org.freedesktop.IBus.Engine"
+IBUS_IFACE_ENGINE_FACTORY   = "org.freedesktop.IBus.EngineFactory"
+IBUS_IFACE_INPUT_CONTEXT    = "org.freedesktop.IBus.InputContext"
+IBUS_IFACE_NOTIFICATIONS    = "org.freedesktop.IBus.Notifications"
 
 def default_reply_handler( *args):
     pass
@@ -94,10 +103,7 @@ CONFIG_GENERAL_SHORTCUT_TRIGGER_DEFAULT = [
     "Zenkaku_Hankaku",
     "Hangul"]
 CONFIG_GENERAL_SHORTCUT_NEXT_ENGINE = "/general/keyboard_shortcut_next_engine"
-CONFIG_GENERAL_SHORTCUT_NEXT_ENGINE_DEFAULT = [
-    "Ctrl+Shift+Release+Shift_L",
-    "Ctrl+Shift+Release+Shift_R",
-    ]
+CONFIG_GENERAL_SHORTCUT_NEXT_ENGINE_DEFAULT = []
 CONFIG_GENERAL_SHORTCUT_PREV_ENGINE = "/general/keyboard_shortcut_prev_engine"
 CONFIG_GENERAL_SHORTCUT_PREV_ENGINE_DEFAULT = []
 
