@@ -27,11 +27,14 @@ __all__ = (
 import gobject
 import gtk
 from gtk import gdk
+from gtk import keysyms
 
 from gettext import dgettext
 _  = lambda a : dgettext("ibus", a)
 N_ = lambda a : a
 
+MAX_HOTKEY = 6
+
 class KeyboardShortcutSelection(gtk.VBox):
     def __init__(self, shortcuts = None):
         super(KeyboardShortcutSelection, self).__init__()
@@ -45,16 +48,17 @@ class KeyboardShortcutSelection(gtk.VBox):
         # self.pack_start(label, False, True, 4)
 
         # shortcuts view
-        viewport =  gtk.Viewport()
-        viewport.set_shadow_type(gtk.SHADOW_IN)
         self.__shortcut_view = gtk.TreeView(gtk.ListStore(gobject.TYPE_STRING))
         self.__shortcut_view.set_size_request(-1, 100)
         renderer = gtk.CellRendererText()
         column = gtk.TreeViewColumn(_("Keyboard shortcuts"), renderer, text = 0)
         self.__shortcut_view.append_column(column)
         self.__shortcut_view.connect("cursor-changed", self.__shortcut_view_cursor_changed_cb)
-        viewport.add(self.__shortcut_view)
-        self.pack_start(viewport, True, True, 4)
+        scrolledwindow = gtk.ScrolledWindow()
+        scrolledwindow.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
+        scrolledwindow.add(self.__shortcut_view)
+        scrolledwindow.set_shadow_type(gtk.SHADOW_IN)
+        self.pack_start(scrolledwindow, True, True, 4)
 
         # key code
         hbox = gtk.HBox()
@@ -80,7 +84,7 @@ class KeyboardShortcutSelection(gtk.VBox):
 
         table = gtk.Table(4, 2)
         self.__modifier_buttons = []
-        self.__modifier_buttons.append(("Ctrl",     gtk.CheckButton("_Ctrl"),       gdk.CONTROL_MASK))
+        self.__modifier_buttons.append(("Control",     gtk.CheckButton("_Control"),       gdk.CONTROL_MASK))
         self.__modifier_buttons.append(("Alt",      gtk.CheckButton("A_lt"),        gdk.MOD1_MASK))
         self.__modifier_buttons.append(("Shift",    gtk.CheckButton("_Shift"),      gdk.SHIFT_MASK))
         self.__modifier_buttons.append(("Meta",     gtk.CheckButton("_Meta"),       gdk.META_MASK))
@@ -123,16 +127,23 @@ class KeyboardShortcutSelection(gtk.VBox):
             shortcuts = []
         model = self.__shortcut_view.get_model()
         model.clear()
+
+        added = []
         for shortcut in shortcuts:
-            model.insert(0, (shortcut,))
-        # self.__shortcut_view.set_model(model)
+            if shortcut not in added:
+                model.insert(-1, (shortcut,))
+                added.append(shortcut)
 
     def get_shortcuts(self):
         model = self.__shortcut_view.get_model()
-        return map(lambda i:i[0] , model)
+        return [i[0] for i in model]
 
     def add_shortcut(self, shortcut):
         model = self.__shortcut_view.get_model()
+        if len(model) > MAX_HOTKEY:
+            return
+        if shortcut in self.get_shortcuts():
+            return
         model.insert(-1, (shortcut,))
 
     def __get_shortcut_from_buttons(self):
@@ -185,19 +196,25 @@ class KeyboardShortcutSelection(gtk.VBox):
         else:
             self.__delete_button.set_sensitive(False)
 
-    def __modifier_button_toggled_cb(self, button, name):
+    def __update_add_and_apply_buttons(self):
         shortcut = self.__get_shortcut_from_buttons()
         selected_shortcut = self.__get_selected_shortcut()
-        self.__add_button.set_sensitive(shortcut != None)
-        can_apply = shortcut != selected_shortcut and shortcut != None and selected_shortcut != None
+        shortcuts = self.get_shortcuts()
+        can_add = shortcut != None and \
+                  shortcut not in shortcuts \
+                  and len(shortcuts) < MAX_HOTKEY
+        self.__add_button.set_sensitive(can_add)
+        can_apply = shortcut != selected_shortcut and \
+                    shortcut != None and \
+                    selected_shortcut != None and \
+                    shortcut not in shortcuts
         self.__apply_button.set_sensitive(can_apply)
 
+    def __modifier_button_toggled_cb(self, button, name):
+        self.__update_add_and_apply_buttons()
+
     def __keycode_entry_notify_cb(self, entry, arg):
-        shortcut = self.__get_shortcut_from_buttons()
-        selected_shortcut = self.__get_selected_shortcut()
-        self.__add_button.set_sensitive(shortcut != None)
-        can_apply = shortcut != selected_shortcut and shortcut != None and selected_shortcut != None
-        self.__apply_button.set_sensitive(can_apply)
+        self.__update_add_and_apply_buttons()
 
     def __keycode_button_clicked_cb(self, button):
         out = []
@@ -216,8 +233,29 @@ class KeyboardShortcutSelection(gtk.VBox):
         if id != gtk.RESPONSE_OK or not out:
             return
         keyevent = out[0]
+        state = keyevent.state & (gdk.CONTROL_MASK | \
+                                  gdk.SHIFT_MASK   | \
+                                  gdk.MOD1_MASK    | \
+                                  gdk.META_MASK    | \
+                                  gdk.SUPER_MASK   | \
+                                  gdk.HYPER_MASK)
+        if state == 0:
+            state = state | gdk.RELEASE_MASK
+        elif keyevent.keyval in (keysyms.Control_L, keysyms.Control_R) and state == gdk.CONTROL_MASK:
+            state = state | gdk.RELEASE_MASK
+        elif keyevent.keyval in (keysyms.Shift_L, keysyms.Shift_R) and state == gdk.SHIFT_MASK:
+            state = state | gdk.RELEASE_MASK
+        elif keyevent.keyval in (keysyms.Alt_L, keysyms.Alt_R) and state == gdk.MOD1_MASK:
+            state = state | gdk.RELEASE_MASK
+        elif keyevent.keyval in (keysyms.Meta_L, keysyms.Meta_R) and state == gdk.META_MASK:
+            state = state | gdk.RELEASE_MASK
+        elif keyevent.keyval in (keysyms.Super_L, keysyms.Super_R) and state == gdk.SUPER_MASK:
+            state = state | gdk.RELEASE_MASK
+        elif keyevent.keyval in (keysyms.Hyper_L, keysyms.Hyper_R) and state == gdk.HYPER_MASK:
+            state = state | gdk.RELEASE_MASK
+
         for name, button, mask in self.__modifier_buttons:
-            if keyevent.state & mask:
+            if state & mask:
                 button.set_active(True)
             else:
                 button.set_active(False)
@@ -258,7 +296,7 @@ if __name__ == "__main__":
     dlg = KeyboardShortcutSelectionDialog(
         title = "Select test",
         buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OK, gtk.RESPONSE_OK))
-    dlg.add_shortcut("Ctrl+Shift+space")
+    dlg.add_shortcut("Control+Shift+space")
     dlg.set_shortcuts(None)
     print dlg.run()
     print dlg.get_shortcuts()
