@@ -19,31 +19,51 @@
 # Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 # Boston, MA  02111-1307  USA
 
+import operator
 import gtk
 import gtk.gdk as gdk
 import gobject
 import pango
 import ibus
-from ibus.gtk import PangoAttrList
+from ibus._gtk import PangoAttrList
 
-class Label(gtk.Label): pass
+class EventBox(gtk.EventBox):
+    def __init__(self):
+        super(EventBox, self).__init__()
+        self.connect("realize", self.__realize_cb)
+
+    def __realize_cb(self, widget):
+        widget.window.set_cursor(gdk.Cursor(gdk.HAND2))
+
+gobject.type_register(EventBox, "IBusEventBox")
+
+class Label(gtk.Label):
+    pass
 gobject.type_register(Label, "IBusPanelLabel")
 
 class HSeparator(gtk.HBox):
-    def __init__ (self):
-        gtk.HBox.__init__ (self)
+    def __init__(self):
+        super(HSeparator, self).__init__()
         self.pack_start(gtk.HSeparator(), True, True, 4)
 
 class VSeparator(gtk.VBox):
-    def __init__ (self):
-        gtk.VBox.__init__ (self)
+    def __init__(self):
+        super(VSeparator, self).__init__()
         self.pack_start(gtk.VSeparator(), True, True, 4)
 
 class CandidateArea(gtk.HBox):
+    __gsignals__ = {
+        "candidate-clicked" : (
+            gobject.SIGNAL_RUN_FIRST,
+            gobject.TYPE_NONE,
+            (gobject.TYPE_UINT, gobject.TYPE_UINT, gobject.TYPE_UINT )),
+    }
+
     def __init__ (self, orientation):
         gtk.HBox.__init__ (self)
         self.__orientation = orientation
         self.__labels = []
+        self.__candidates = []
         self.__create_ui()
 
     def __create_ui(self):
@@ -56,35 +76,63 @@ class CandidateArea(gtk.HBox):
             self.pack_start(VSeparator(), False, False, 0)
             self.pack_start(self.__vbox2, True, True, 4)
 
-        for i in xrange(1, 11):
-            label1 = Label("%d." % (i % 10))
+        for i in range(0, 16):
+            label1 = Label("%c." % ("1234567890abcdef"[i]))
             label1.set_alignment(0.0, 0.5)
-            label1.set_no_show_all(True)
+            label1.show()
 
             label2 = Label()
             label2.set_alignment(0.0, 0.5)
-            label2.set_no_show_all(True)
+            label1.show()
 
             if self.__orientation == gtk.ORIENTATION_VERTICAL:
                 label1.set_property("xpad", 8)
                 label2.set_property("xpad", 8)
-                self.__vbox1.pack_start(label1, False, False, 2)
-                self.__vbox2.pack_start(label2, False, False, 2)
+                ebox1 = EventBox()
+                ebox1.set_no_show_all(True)
+                ebox1.add(label1)
+                ebox2 = EventBox()
+                ebox2.set_no_show_all(True)
+                ebox2.add(label2)
+                self.__vbox1.pack_start(ebox1, False, False, 2)
+                self.__vbox2.pack_start(ebox2, False, False, 2)
+                self.__candidates.append((ebox1, ebox2))
             else:
                 hbox = gtk.HBox()
+                hbox.show()
                 hbox.pack_start(label1, False, False, 1)
                 hbox.pack_start(label2, False, False, 1)
-                self.pack_start(hbox, False, False, 4)
+                ebox = EventBox()
+                ebox.set_no_show_all(True)
+                ebox.add(hbox)
+                self.pack_start(ebox, False, False, 4)
+                self.__candidates.append((ebox,))
 
             self.__labels.append((label1, label2))
 
-        self.__labels[0][0].show()
-        self.__labels[0][1].show()
+        cursor = gdk.Cursor(gdk.HAND1)
+        for i, ws in enumerate(self.__candidates):
+            for w in ws:
+                w.connect("button-press-event", lambda w, e, i:self.emit("candidate-clicked", i, e.button, e.state), i)
+
+    def set_labels(self, labels):
+        if not labels:
+            for i in xrange(0, 16):
+                self.__labels[i][0].set_text("%c." % ("1234567890abcdef"[i]))
+                self.__labels[i][0].set_property("attributes", None)
+            return
+
+        i = 0
+        for text, attrs in labels:
+            self.__labels[i][0].set_text(text)
+            self.__labels[i][0].set_property("attributes", attrs)
+            i += 1
+            if i >= 16:
+                break
 
     def set_candidates(self, candidates, focus_candidate = 0, show_cursor = True):
         assert len(candidates) <= len(self.__labels)
-        i = 0
-        for text, attrs in candidates:
+        for i, (text, attrs) in enumerate(candidates):
             if i == focus_candidate and show_cursor:
                 if attrs == None:
                     attrs = pango.AttrList()
@@ -97,21 +145,13 @@ class CandidateArea(gtk.HBox):
                 attrs.insert(attr)
 
             self.__labels[i][1].set_text(text)
-            self.__labels[i][1].set_property("attributes", attrs)
-            self.__labels[i][0].show()
             self.__labels[i][1].show()
+            self.__labels[i][1].set_property("attributes", attrs)
+            for w in self.__candidates[i]:
+                w.show()
 
-            i += 1
-
-        for label1, label2 in self.__labels[max(1, len(candidates)):]:
-            label1.hide()
-            label2.hide()
-
-        if len(candidates) == 0:
-            self.__labels[0][0].set_text("")
-            self.__labels[0][1].set_text("")
-        else:
-            self.__labels[0][0].set_text("1.")
+        for w in reduce(operator.add, self.__candidates[len(candidates):]):
+            w.hide()
 
 class CandidatePanel(gtk.VBox):
     __gproperties__ = {
@@ -131,6 +171,18 @@ class CandidatePanel(gtk.VBox):
             gobject.SIGNAL_RUN_FIRST,
             gobject.TYPE_NONE,
             ()),
+        "page-up" : (
+            gobject.SIGNAL_RUN_FIRST,
+            gobject.TYPE_NONE,
+            ()),
+        "page-down" : (
+            gobject.SIGNAL_RUN_FIRST,
+            gobject.TYPE_NONE,
+            ()),
+        "candidate-clicked" : (
+            gobject.SIGNAL_RUN_FIRST,
+            gobject.TYPE_NONE,
+            (gobject.TYPE_UINT, gobject.TYPE_UINT, gobject.TYPE_UINT)),
     }
 
     def __init__ (self):
@@ -193,6 +245,7 @@ class CandidatePanel(gtk.VBox):
         # create candidates area
         self.__candidate_area = CandidateArea(self.__orientation)
         self.__candidate_area.set_no_show_all(True)
+        self.__candidate_area.connect("candidate-clicked", lambda x, i, b, s: self.emit("candidate-clicked", i, b, s))
         self.update_lookup_table(self.__lookup_table, self.__lookup_table_visible)
 
         # create state label
@@ -201,14 +254,14 @@ class CandidatePanel(gtk.VBox):
 
         # create buttons
         self.__prev_button = gtk.Button()
-        self.__prev_button.connect("clicked", lambda x: self.emit("cursor-up"))
+        self.__prev_button.connect("clicked", lambda x: self.emit("page-up"))
         self.__prev_button.set_relief(gtk.RELIEF_NONE)
-        self.__tooltips.set_tip(self.__prev_button, "Previous candidate")
+        self.__tooltips.set_tip(self.__prev_button, "Previous page")
 
         self.__next_button = gtk.Button()
-        self.__next_button.connect("clicked", lambda x: self.emit("cursor-down"))
+        self.__next_button.connect("clicked", lambda x: self.emit("page-down"))
         self.__next_button.set_relief(gtk.RELIEF_NONE)
-        self.__tooltips.set_tip(self.__next_button, "Next candidate")
+        self.__tooltips.set_tip(self.__next_button, "Next page")
 
         self.__pack_all_widgets()
 
@@ -222,6 +275,7 @@ class CandidatePanel(gtk.VBox):
             image = gtk.Image()
             image.set_from_stock(gtk.STOCK_GO_DOWN, gtk.ICON_SIZE_MENU)
             self.__next_button.set_image(image)
+
             vbox = gtk.VBox()
             vbox.pack_start(self.__preedit_label, False, False, 0)
             vbox.pack_start(self.__aux_label, False, False, 0)
@@ -238,11 +292,11 @@ class CandidatePanel(gtk.VBox):
         else:
             # package all widgets in HORIZONTAL mode
             image = gtk.Image()
-            image.set_from_stock(gtk.STOCK_GO_BACK, gtk.ICON_SIZE_MENU)
+            image.set_from_stock(gtk.STOCK_GO_UP, gtk.ICON_SIZE_MENU)
             self.__prev_button.set_image(image)
 
             image = gtk.Image()
-            image.set_from_stock(gtk.STOCK_GO_FORWARD, gtk.ICON_SIZE_MENU)
+            image.set_from_stock(gtk.STOCK_GO_DOWN, gtk.ICON_SIZE_MENU)
             self.__next_button.set_image(image)
 
             vbox = gtk.VBox()
@@ -260,58 +314,62 @@ class CandidatePanel(gtk.VBox):
         # self.hide_all()
         # self.show_all()
 
-    def show_preedit(self):
+    def show_preedit_text(self):
         self.__preedit_visible = True
         self.__preedit_label.show()
         self.__check_show_states()
 
-    def hide_preedit(self):
+    def hide_preedit_text(self):
         self.__preedit_visible = False
         self.__preedit_label.hide()
         self.__check_show_states()
 
-    def update_preedit(self, text, attrs, cursor_pos, visible):
-        if attrs:
-            attrs = PangoAttrList(attrs, text)
+    def update_preedit_text(self, text, cursor_pos, visible):
+        attrs = PangoAttrList(text.attributes, text.text)
         if visible:
-            self.show_preedit()
+            self.show_preedit_text()
         else:
-            self.hide_preedit()
-        self.__preedit_string = text
-        self.__preedit_label.set_text(text)
-        if attrs == None:
-            attrs = pango.AttrList()
+            self.hide_preedit_text()
+        self.__preedit_stribg = text.text
+        self.__preedit_label.set_text(text.text)
         self.__preedit_attrs = attrs
         self.__preedit_label.set_attributes(attrs)
 
-    def show_aux_string(self):
+    def show_auxiliary_text(self):
         self.__aux_string_visible = True
         self.__aux_label.show()
         self.__check_show_states()
 
-    def hide_aux_string(self):
+    def hide_auxiliary_text(self):
         self.__aux_string_visible = False
         self.__aux_label.hide()
         self.__check_show_states()
 
-    def update_aux_string(self, text, attrs, show):
-        attrs = PangoAttrList(attrs, text)
+    def update_auxiliary_text(self, text, show):
+        attrs = PangoAttrList(text.attributes, text.text)
 
         if show:
-            self.show_aux_string()
+            self.show_auxiliary_text()
         else:
-            self.hide_aux_string()
+            self.hide_auxiliary_text()
 
-        self.__aux_string = text
-        self.__aux_label.set_text(text)
-        if attrs == None:
-            attrs = pango.AttrList()
+        self.__aux_string = text.text
+        self.__aux_label.set_text(text.text)
         self.__aux_attrs = attrs
         self.__aux_label.set_attributes(attrs)
 
+    def __refresh_labels(self):
+        labels = self.__lookup_table.get_labels()
+        if labels:
+            labels = map(lambda x: (x.text, PangoAttrList(x.attributes, x.text)), labels)
+        else:
+            labels = None
+        self.__candidate_area.set_labels(labels)
+
+
     def __refresh_candidates(self):
-        candidates = self.__lookup_table.get_canidates_in_current_page()
-        candidates = map(lambda x: (x[0], PangoAttrList(x[1], x[0]) if x[1] else None), candidates)
+        candidates = self.__lookup_table.get_candidates_in_current_page()
+        candidates = map(lambda x: (x.text, PangoAttrList(x.attributes, x.text)), candidates)
         self.__candidate_area.set_candidates(candidates,
                 self.__lookup_table.get_cursor_pos_in_current_page(),
                 self.__lookup_table.is_cursor_visible()
@@ -328,6 +386,7 @@ class CandidatePanel(gtk.VBox):
 
         self.__lookup_table = lookup_table
         self.__refresh_candidates()
+        self.__refresh_labels()
 
     def show_lookup_table(self):
         self.__lookup_table_visible = True
@@ -372,8 +431,8 @@ class CandidatePanel(gtk.VBox):
             self.emit("hide")
 
     def reset(self):
-        self.update_preedit("", None, 0, False)
-        self.update_aux_string("", None, False)
+        self.update_preedit_text(ibus.Text(""), 0, False)
+        self.update_auxiliary_text(ibus.Text(""), False)
         self.update_lookup_table(None, False)
         self.hide()
 
